"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var numberMeasure_1 = require("../numberMeasure");
describe("Number measures", function () {
    var meters = numberMeasure_1.Measure.dimension("length", "m");
    var seconds = numberMeasure_1.Measure.dimension("time", "s");
    var mps = meters.per(seconds);
    var mps2 = mps.per(seconds);
    describe("dimension", function () {
        it("should create dimensions with value 1", function () {
            expect(numberMeasure_1.Measure.dimension("foo", "f")).toEqual({ value: 1, unit: { foo: ["f", 1] }, symbol: "f" });
        });
    });
    describe("construction", function () {
        it("should construct from a number of and a unit", function () {
            var measure = numberMeasure_1.Measure.of(10, mps2);
            expect(measure.value).toBe(10);
            expect(measure.unit).toEqual(mps2.unit);
        });
        it("should construct from a number and another measure", function () {
            var kilometer = numberMeasure_1.Measure.of(1000, meters);
            var measure = numberMeasure_1.Measure.of(5.2, kilometer);
            expect(measure.value).toBe(5200);
            expect(measure.unit).toEqual(meters.unit);
        });
        it("should construct dimensionless values", function () {
            var dimensionless = numberMeasure_1.Measure.dimensionless(3);
            expect(dimensionless.value).toBe(3);
            expect(dimensionless.unit).toEqual({});
        });
    });
    describe("prefixes", function () {
        var kilo = numberMeasure_1.Measure.prefix("k", 1000);
        it("should scale the base unit", function () {
            var km = kilo(meters);
            expect(km.unit).toEqual(meters.unit);
            expect(km.value).toBe(1000);
        });
        it("should apply a prefix when a symbol is present on the base unit", function () {
            expect(kilo(meters).symbol).toBe("km");
        });
        it("should not apply a prefix when a symbol is not present on the base unit", function () {
            var blargs = numberMeasure_1.Measure.of(1e-3, meters);
            var kblargs = kilo(blargs);
            expect(kblargs.symbol).toBeUndefined();
            expect(kblargs.value).toBe(1);
        });
    });
    describe("math", function () {
        it("arithmetic", function () {
            expect(numberMeasure_1.Measure.add(numberMeasure_1.Measure.of(5, mps), numberMeasure_1.Measure.of(-5, mps))).toEqual(numberMeasure_1.Measure.of(0, mps));
            expect(numberMeasure_1.Measure.subtract(numberMeasure_1.Measure.of(5, mps), numberMeasure_1.Measure.of(-5, mps))).toEqual(numberMeasure_1.Measure.of(10, mps));
            expect(numberMeasure_1.Measure.multiply(numberMeasure_1.Measure.of(5, mps), numberMeasure_1.Measure.of(10, seconds))).toEqual(numberMeasure_1.Measure.of(50, meters));
            expect(numberMeasure_1.Measure.divide(numberMeasure_1.Measure.of(50, meters), numberMeasure_1.Measure.of(10, seconds))).toEqual(numberMeasure_1.Measure.of(5, mps));
        });
        it("abs", function () {
            expect(numberMeasure_1.Measure.abs(numberMeasure_1.Measure.of(-10, mps))).toEqual(numberMeasure_1.Measure.of(10, mps));
        });
        it("cbrt", function () {
            expect(numberMeasure_1.Measure.cbrt(numberMeasure_1.Measure.of(64, seconds.cubed()))).toEqual(numberMeasure_1.Measure.of(4, seconds));
        });
        it("ceil", function () {
            expect(numberMeasure_1.Measure.ceil(numberMeasure_1.Measure.of(3.4, mps))).toEqual(numberMeasure_1.Measure.of(4, mps));
        });
        it("floor", function () {
            expect(numberMeasure_1.Measure.floor(numberMeasure_1.Measure.of(7.8, mps))).toEqual(numberMeasure_1.Measure.of(7, mps));
        });
        it("hypot", function () {
            expect(numberMeasure_1.Measure.hypot(numberMeasure_1.Measure.of(3, meters), numberMeasure_1.Measure.of(4, meters))).toEqual(numberMeasure_1.Measure.of(5, meters));
        });
        it("max", function () {
            expect(numberMeasure_1.Measure.max(numberMeasure_1.Measure.of(10, mps), numberMeasure_1.Measure.of(5, mps), numberMeasure_1.Measure.of(15, mps))).toEqual(numberMeasure_1.Measure.of(15, mps));
        });
        it("min", function () {
            expect(numberMeasure_1.Measure.min(numberMeasure_1.Measure.of(10, mps), numberMeasure_1.Measure.of(5, mps), numberMeasure_1.Measure.of(15, mps))).toEqual(numberMeasure_1.Measure.of(5, mps));
        });
        it("round", function () {
            expect(numberMeasure_1.Measure.round(numberMeasure_1.Measure.of(7.8, mps))).toEqual(numberMeasure_1.Measure.of(8, mps));
        });
        it("sqrt", function () {
            expect(numberMeasure_1.Measure.sqrt(numberMeasure_1.Measure.of(25, meters.squared()))).toEqual(numberMeasure_1.Measure.of(5, meters));
        });
        it("sum", function () {
            expect(numberMeasure_1.Measure.sum(numberMeasure_1.Measure.of(10, mps), numberMeasure_1.Measure.of(5, mps), numberMeasure_1.Measure.of(15, mps))).toEqual(numberMeasure_1.Measure.of(30, mps));
        });
        it("trunc", function () {
            expect(numberMeasure_1.Measure.trunc(numberMeasure_1.Measure.of(-7.8, mps))).toEqual(numberMeasure_1.Measure.of(-7, mps));
        });
    });
    describe("arithmetic", function () {
        it("should negate", function () {
            var value = numberMeasure_1.Measure.of(10, mps);
            expect(value.negate()).toEqual(numberMeasure_1.Measure.of(-10, mps));
        });
        it("should add", function () {
            var left = numberMeasure_1.Measure.of(10, mps);
            var right = numberMeasure_1.Measure.of(5, mps);
            expect(left.plus(right)).toEqual(numberMeasure_1.Measure.of(15, mps));
        });
        it("should subtract", function () {
            var left = numberMeasure_1.Measure.of(10, seconds);
            var right = numberMeasure_1.Measure.of(5, seconds);
            expect(left.minus(right)).toEqual(numberMeasure_1.Measure.of(5, seconds));
        });
        it("should multiply", function () {
            var left = numberMeasure_1.Measure.of(10, mps);
            var right = numberMeasure_1.Measure.of(5, seconds);
            expect(left.times(right)).toEqual(numberMeasure_1.Measure.of(50, meters));
        });
        it("should divide", function () {
            var left = numberMeasure_1.Measure.of(10, mps);
            var right = numberMeasure_1.Measure.of(5, seconds);
            expect(left.over(right)).toEqual(numberMeasure_1.Measure.of(2, mps2));
            expect(left.per(right)).toEqual(numberMeasure_1.Measure.of(2, mps2));
            expect(left.div(right)).toEqual(numberMeasure_1.Measure.of(2, mps2));
        });
        it("should scale", function () {
            var value = numberMeasure_1.Measure.of(10, mps);
            expect(value.scale(2)).toEqual(numberMeasure_1.Measure.of(20, mps));
        });
        it("should exponentiate", function () {
            var value = numberMeasure_1.Measure.of(10, meters);
            expect(value.inverse()).toEqual(numberMeasure_1.Measure.of(0.1, meters.inverse()));
            expect(value.reciprocal()).toEqual(numberMeasure_1.Measure.of(0.1, meters.inverse()));
            expect(value.toThe(0)).toEqual(numberMeasure_1.Measure.dimensionless(1));
            expect(value.toThe(1)).toEqual(numberMeasure_1.Measure.of(10, meters));
            expect(value.squared()).toEqual(numberMeasure_1.Measure.of(100, meters.squared()));
            expect(value.cubed()).toEqual(numberMeasure_1.Measure.of(1000, meters.cubed()));
        });
    });
    describe("comparison", function () {
        var zero = numberMeasure_1.Measure.of(0, meters);
        var five = numberMeasure_1.Measure.of(5, meters);
        var ten = numberMeasure_1.Measure.of(10, meters);
        it("should compare less than", function () {
            expect(five.lt(zero)).toBe(false);
            expect(five.lt(five)).toBe(false);
            expect(five.lt(ten)).toBe(true);
        });
        it("should compare less than or equal to", function () {
            expect(five.lte(zero)).toBe(false);
            expect(five.lte(five)).toBe(true);
            expect(five.lte(ten)).toBe(true);
        });
        it("should compare equal to", function () {
            expect(five.eq(zero)).toBe(false);
            expect(five.eq(five)).toBe(true);
            expect(five.eq(ten)).toBe(false);
        });
        it("should compare not equal to", function () {
            expect(five.neq(zero)).toBe(true);
            expect(five.neq(five)).toBe(false);
            expect(five.neq(ten)).toBe(true);
        });
        it("should compare greater than or equal to", function () {
            expect(five.gte(zero)).toBe(true);
            expect(five.gte(five)).toBe(true);
            expect(five.gte(ten)).toBe(false);
        });
        it("should compare greater than", function () {
            expect(five.gt(zero)).toBe(true);
            expect(five.gt(five)).toBe(false);
            expect(five.gt(ten)).toBe(false);
        });
    });
    describe("symbols", function () {
        it("should assign a symbol via .of()", function () {
            expect(numberMeasure_1.Measure.of(1000, meters, "km").symbol).toBe("km");
        });
        it("should copy assign a symbol via .withSymbol()", function () {
            var original = numberMeasure_1.Measure.of(1000, meters);
            var result = original.withSymbol("km");
            expect(result).not.toBe(original);
            expect(original.symbol).toBeUndefined();
            expect(result.symbol).toBe("km");
        });
        it("should not pass along symbols through operations", function () {
            var km = numberMeasure_1.Measure.of(1000, meters.squared()).withSymbol("km2");
            var dm = numberMeasure_1.Measure.of(10, meters.squared()).withSymbol("dm2");
            expect(km.negate().symbol).toBeUndefined();
            expect(km.squared().symbol).toBeUndefined();
            expect(km.inverse().symbol).toBeUndefined();
            expect(km.plus(dm).symbol).toBeUndefined();
            expect(km.minus(dm).symbol).toBeUndefined();
            expect(km.times(dm).symbol).toBeUndefined();
            expect(km.over(dm).symbol).toBeUndefined();
        });
    });
    describe("formatting", function () {
        it("should format dimensionless units", function () {
            expect(numberMeasure_1.Measure.dimensionless(10).toString()).toBe("10");
        });
        it("should format base units", function () {
            expect(meters.toString()).toBe("1 m");
            expect(numberMeasure_1.Measure.of(5.3, meters).toString()).toBe("5.3 m");
        });
        it("should format complex units", function () {
            expect(numberMeasure_1.Measure.of(5, meters.squared()).toString()).toBe("5 m^2");
            expect(numberMeasure_1.Measure.of(5, seconds.inverse()).toString()).toBe("5 s^-1");
            expect(numberMeasure_1.Measure.of(5, meters.times(seconds)).toString()).toBe("5 m * s");
            expect(numberMeasure_1.Measure.of(5, meters.over(seconds)).toString()).toBe("5 m * s^-1");
            expect(numberMeasure_1.Measure.of(5, meters.cubed().over(seconds)).toString()).toBe("5 m^3 * s^-1");
            expect(numberMeasure_1.Measure.of(5, meters.cubed().over(seconds.squared())).toString()).toBe("5 m^3 * s^-2");
        });
        it("should have consistent formatting no matter how the unit is constructed", function () {
            var metersTimesSecond = "5 m * s";
            expect(numberMeasure_1.Measure.of(5, meters.times(seconds)).toString()).toBe(metersTimesSecond);
            expect(numberMeasure_1.Measure.of(5, seconds.times(meters)).toString()).toBe(metersTimesSecond);
            var metersPerSecond = "5 m * s^-1";
            expect(numberMeasure_1.Measure.of(5, meters.per(seconds)).toString()).toBe(metersPerSecond);
            expect(numberMeasure_1.Measure.of(5, seconds.inverse().times(meters)).toString()).toBe(metersPerSecond);
        });
        it("should not format using symbol even if present", function () {
            expect(numberMeasure_1.Measure.of(5, meters.squared())
                .withSymbol("m2")
                .toString()).toBe("5 m^2");
            expect(numberMeasure_1.Measure.dimensionless(0)
                .withSymbol("rad")
                .toString()).toBe("0");
        });
        it("should format measures as other measures with symbols", function () {
            var glorbs = numberMeasure_1.Measure.of(100, meters, "glb");
            expect(numberMeasure_1.Measure.of(1000, meters).in(glorbs)).toBe("10 glb");
        });
        it("should use normal formatting if the other measure has no symbol", function () {
            var glorbs = numberMeasure_1.Measure.of(100, meters);
            expect(numberMeasure_1.Measure.of(1000, meters).in(glorbs)).toBe("1000 m");
        });
        it("should use base unit symbols to format when available", function () {
            var m = numberMeasure_1.Measure.dimension("test-length", "meter");
            var s = numberMeasure_1.Measure.dimension("test-time", "second");
            expect(m.toString()).toBe("1 meter");
            expect(numberMeasure_1.Measure.of(1, m.per(s)).toString()).toBe("1 meter * second^-1");
            expect(numberMeasure_1.Measure.of(1, m.squared().per(s.squared())).toString()).toBe("1 meter^2 * second^-2");
        });
    });
});
//# sourceMappingURL=numberMeasureTests.js.map