import { GenericMeasure } from "./genericMeasure";
import { Unit } from "./unitTypeArithmetic";
/** A function which applies a symbol prefix and multiplier to a given measure. */
export declare type PrefixFunction<N> = <U extends Unit>(measure: GenericMeasure<N, U>) => GenericMeasure<N, U>;
/** A function which transforms a single measure into another measure with the same unit. */
export declare type UnaryMeasureFunction<N> = <U extends Unit>(x: GenericMeasure<N, U>) => GenericMeasure<N, U>;
/** A function which transforms two measures with same unit into a single measure with the same unit. */
export declare type BinaryMeasureFunction<N> = <U extends Unit>(left: GenericMeasure<N, U>, right: GenericMeasure<N, U>) => GenericMeasure<N, U>;
/** A function which transforms one or more measure with the same unit into a single measure with the same unit. */
export declare type SpreadMeasureFunction<N> = <U extends Unit>(first: GenericMeasure<N, U>, ...rest: Array<GenericMeasure<N, U>>) => GenericMeasure<N, U>;
/**
 * Converts a unary function of unitless numbers into a function of measures. This assumes that the underlying
 * operation makes no change to the unit of the measure. For example, this would be an incorrect usage:
 * `square = wrapUnaryFn((value: number) => value ** 2)` since squaring a measure would change its unit.
 * @param fn a unary function of numeric types
 * @returns a unary function of measures
 */
export declare function wrapUnaryFn<N>(fn: (x: N) => N): UnaryMeasureFunction<N>;
/**
 * Converts a binary function of unitless numbers into a function of measures. This assumes that the underlying
 * operation makes no change to the unit of the measure. For example, this would be an incorrect usage:
 * `mult = wrapBinaryFn((left, right) => left * right)` since multiplying two measures would result in a different unit.
 * @param fn a binary function of numeric types
 * @returns a binary function of measures
 */
export declare function wrapBinaryFn<N>(fn: (left: N, right: N) => N): BinaryMeasureFunction<N>;
/**
 * Converts a function of any number of unitless numbers into a function of one or more measures. This assumes that the
 * underlying operation makes no change to the unit of the measure. For example, this would be an incorrect usage:
 * `product = wrapBinaryFn((...values) => product(values))` since multiplying measures would result in a different unit.
 * Note that the resulting function requires at least one value in order to compute the unit of the resulting measure.
 * @param fn a spread function of numeric types
 * @returns a spread function of measures
 */
export declare function wrapSpreadFn<N>(fn: (...x: N[]) => N): SpreadMeasureFunction<N>;
